//////////////////////////////////////////////////////////////////////////////
// Module	: ClsNonStd.cpp													//
// Function	: Base class for all ISA/PCI/PC104/Ethernet Smart A/D boards.	//
// Author	: Jim Lamberson													//
//////////////////////////////////////////////////////////////////////////////

#include "ClsNonStd.h"		// Class declaration.

//////////////////////////////////////////////////////////////////////////////
// Constructor.

ClsNonStd::ClsNonStd( short ProductID, long Adrs, short NumCh, const char *pSdcTbl, const char *pCmdTbl, char NumBoards ) :
	ClsSmad( ProductID, Adrs, NumCh, pSdcTbl, pCmdTbl, NumBoards )
{
	// Base class does all initialization.
}

//////////////////////////////////////////////////////////////////////////////
// Fetch product ID from Smart A/D board.

short ClsNonStd::ProductIDGet()
{
	CmdOpen();
	SendByte( Cmd[CMD_DIAGNOSE] );
	SendByte( DIAG_GETPRODUCTID );
	SendByte( 0 );
	DeclareResponse( 2 );
	CmdClose();
	return ReadWord();
}

///////////////////////////////////////////////////////////////////////////
// Program sensor type.

void ClsNonStd::SensorTypeSet( short Chan, short LogSDC )
{
	// If channel number is valid ...
	if ( IsValidChan( Chan ) )
	{
		// If sensor type is in legal range ...
		if ( LogSDC < NUM_SDCS )
		{
			// If sensor type is supported by this product ...
			if ( PhysSDC[LogSDC] != 0xFF )
			{
				// Initialize images of all programmable channel properties.
				short LogChan = Chan;
				InitChanProperties( LogChan, LogSDC );

				// Declare the new sensor type to the Smart A/D board.
				CmdOpen();									// Begin new command.
				SelectBoardByChan( &Chan );					// Select sub-board, if applicable.
				SendByte( Cmd[CMD_DFNTYP] | Chan );			// Issue SetSensorType command.
				SendByte( PhysSDC[LogSDC] );
				CmdClose();									// Execute the command.



				double StartTime = SysTimer();
				while ( ElapsedTime( StartTime ) < 0.005 );




				// Exit.
				return;										// Normal exit.
			}
		}

		// Sensor type is not supported or is outside of the legal value range, so call external error
		// handler if one has been declared.
		pMyErrHandler( ERR_BADPARM );
	}
}

//////////////////////////////////////////////////////////////////////////////
// Set linearization coefficients for custom resistive sensor channel.

void ClsNonStd::CoefficientsSet( short Chan, double c1, double c2, double c3 )
{
	// If channel number is valid ...
	if ( IsValidChan( Chan ) )
	{
		// If this Smart A/D model supports custom resistive sensors ...
		if ( Cmd[CMD_WCOEFF] != 0xFF )
		{
			// Execute SetCoefficients function on Smart A/D board.
			CmdOpen();
			SelectBoardByChan( &Chan );
			SendByte( Cmd[CMD_WCOEFF] | Chan );
			SendReal( c1 );
			SendReal( c2 );
			SendReal( c3 );
			CmdClose();
		}
	}
}

//////////////////////////////////////////////////////////////////////////////
// Read gage calibration from Smart A/D.

void ClsNonStd::GageCalGet( short Chan, double *pSlope, short *pOffset )
{
	// If channel number is valid ...
	if ( IsValidChan( Chan ) )
	{
		// If this Smart A/D model supports gages ...
		if ( Cmd[CMD_RDSLOP] != 0xFF )
		{
			// Execute GetGageCal function on Smart A/D board.
			CmdOpen();
			SelectBoardByChan( &Chan );
			SendByte( Cmd[CMD_RDSLOP] | Chan );
			DeclareResponse( 6 );
			CmdClose();

			// Return gage calibration values to caller.
			*pSlope  = ReadReal();
			*pOffset = ReadWordRev();
		}
		else
		{
			*pSlope  = 0.0;
			*pOffset = 0;
		}
	}
}

//////////////////////////////////////////////////////////////////////////////
// Set channel gage calibration.

#define GCS_RETRIES		3

void ClsNonStd::GageCalSet( short Chan, double Slope, short Offset )
{
	// If channel number is valid ...
	if ( IsValidChan( Chan ) )
	{
		// If this Smart A/D model supports gages ...
		if ( Cmd[CMD_WTSLOP] != 0xFF )
		{
			// Select sub-board to access.
			SelectBoardByChan( &Chan );

			// Repeat until cal values stick or retries are exhausted ...
			for ( int i = 0; i < GCS_RETRIES; i++ )
			{
				// Execute SetGageCal function on Smart A/D board.
				CmdOpen();
				SendByte( Cmd[CMD_WTSLOP] | Chan );
				SendReal( Slope );
				SendWordRev( Offset );
				CmdClose();

				// Delay 22 ms for new values to stick.
				double StartTime = SysTimer();
				while ( ElapsedTime(StartTime) < 0.022 );

				// Execute GetGageCal function on Smart A/D board to get programmed values.
				CmdOpen();
				SendByte( Cmd[CMD_RDSLOP] | Chan );
				DeclareResponse( 6 );
				CmdClose();

				// Normal exit if both cal values stuck (must be tested in order of slope, offset) ...
				if ( Slope == ReadReal() )
				{
					if ( Offset == ReadWordRev() )
						return;
				}
			}

			// Abnormal exit.
			return;
		}
	}
}

//////////////////////////////////////////////////////////////////////////////
// Declare gage full-scale condition.

void ClsNonStd::GageSpanSet( short Chan, short Load )
{
	// If channel number is valid ...
	if ( IsValidChan( Chan ) )
	{
		// If this Smart A/D model supports gages ...
		if ( Cmd[CMD_GSPAN] != 0xFF )
		{
			// Execute SetGageSpan function on Smart A/D board.
			CmdOpen();
			SelectBoardByChan( &Chan );
			SendByte( Cmd[CMD_GSPAN] | Chan );
			SendWord( Load );
			CmdClose();
		}
	}
}

//////////////////////////////////////////////////////////////////////////////
// Tare gage.

void ClsNonStd::GageTareSet( short Chan )
{
	// If channel number is valid ...
	if ( IsValidChan( Chan ) )
	{
		// If this Smart A/D model supports gages ...
		if ( Cmd[CMD_TARE] != 0xFF )
		{
			// Execute TareGage function on Smart A/D board.
			CmdOpen();
			SelectBoardByChan( &Chan );
			SendByte( Cmd[CMD_TARE] | Chan );
			CmdClose();
		}
	}
}

//////////////////////////////////////////////////////////////////////////////
// Declare gage zero-scale condition.

void ClsNonStd::GageZeroSet( short Chan )
{
	// If channel number is valid ...
	if ( IsValidChan( Chan ) )
	{
		// If this Smart A/D model supports gages ...
		if ( Cmd[CMD_GZERO] != 0xFF )
		{
			// Execute SetGageZero function on Smart A/D board.
			CmdOpen();
			SelectBoardByChan( &Chan );
			SendByte( Cmd[CMD_GZERO] | Chan );
			CmdClose();
		}
	}
}
